﻿using System;
using System.Linq;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace SilverlightApplication1
{
    public class NameList : ObservableCollection<PersonName> 
    {
        

        public NameList()
            : base()
        {
            Add(new PersonName("A", "11"));
            Add(new PersonName("B", "22"));
            Add(new PersonName("C", "33"));
            Add(new PersonName("D", "44"));
            Add(new PersonName("E", "55"));
        }

        protected override void InsertItem(int index, PersonName item)
        {
            base.InsertItem(index, item);
            item.PropertyChanged += new PropertyChangedEventHandler(item_PropertyChanged);
        }

        protected override void RemoveItem(int index)
        {
            this[index].PropertyChanged -= item_PropertyChanged;
            base.RemoveItem(index);
        }

        void item_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("IsChecked"))
            {
                this.HasCheckedItems = this.Any(i => i.IsChecked == true);
            }
        }


        private bool hasCheckedItems;
        /// <summary>
        ///     Gets or sets whether there are any checked items.
        /// </summary>
        public bool HasCheckedItems
        {
            get
            {
                return this.hasCheckedItems;
            }
            set
            {
                if (this.hasCheckedItems != value)
                {
                    this.hasCheckedItems = value;
                    this.OnPropertyChanged(new PropertyChangedEventArgs("HasCheckedItems"));
                }
            }
        }
        
        
    }

    public class PersonName : INotifyPropertyChanged
    {
        public PersonName(string first, string last)
        {
            this.firstName = first;
            this.lastName = last;

            this.UpdateIsCheckable();
        }

        private void UpdateIsCheckable()
        {
            this.IsCheckable = "11".Equals(this.lastName);
        }


        private bool isCheckable;
        /// <summary>
        ///     Gets or sets whether the item should be checkable.
        /// </summary>
        public bool IsCheckable
        {
            get
            {
                return this.isCheckable;
            }
            set
            {
                if (this.isCheckable != value)
                {
                    this.isCheckable = value;
                    OnPropertyChanged("IsCheckable");
                }
            }
        }

        private bool isChecked;
        /// <summary>
        ///     Gets or sets whether the item is chedked.
        /// </summary>
        public bool IsChecked
        {
            get
            {
                return this.isChecked;
            }
            set
            {
                if (this.isChecked != value)
                {
                    this.isChecked = value;
                    OnPropertyChanged("IsChecked");
                }
            }
        }
        
        

        private String lastName;
        /// <summary>
        ///     Gets or sets last name.
        /// </summary>
        public String LastName
        {
            get
            {
                return this.lastName;
            }
            set
            {
                if (this.lastName != value)
                {
                    this.lastName = value;
                    OnPropertyChanged("LastName");
                    UpdateIsCheckable();
                }
            }
        }
        

        private String firstName;
        /// <summary>
        ///     Gets or sets first name.
        /// </summary>
        public String FirstName
        {
            get
            {
                return this.firstName;
            }
            set
            {
                if (this.firstName != value)
                {
                    this.firstName = value;
                    OnPropertyChanged("FirstName");
                }
            }
        }
        

        /// <summary>
        ///     Event for INotifyPropertyChanged.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        ///     Raises the INotifyPropertyChanged event.
        /// </summary>
        /// <param name="propertyName">The property to be included in the INotifyPropertyChanged EventArgs.</param>
        protected virtual void OnPropertyChanged(String propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        
    }

}
